<?php

namespace SGalinski\SgMail\Controller;

/***************************************************************
 *  Copyright notice
 *
 *  (c) sgalinski Internet Services (https://www.sgalinski.de)
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use SGalinski\SgMail\Domain\Repository\LayoutRepository;
use SGalinski\SgMail\Service\BackendService;
use SGalinski\SgMail\Session\PhpSession;
use TYPO3\CMS\Backend\Clipboard\Clipboard;
use TYPO3\CMS\Backend\Template\Components\DocHeaderComponent;
use TYPO3\CMS\Backend\Utility\BackendUtility;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Utility\VersionNumberUtility;
use TYPO3\CMS\Extbase\Mvc\Controller\ActionController;
use TYPO3\CMS\Extbase\Mvc\Exception\NoSuchArgumentException;
use TYPO3\CMS\Extbase\Mvc\View\ViewInterface;

/**
 * Controller for the mail templating service module
 */
class LayoutController extends ActionController {
	/**
	 * DocHeaderComponent
	 *
	 * @var DocHeaderComponent
	 */
	protected $docHeaderComponent;

	/**
	 * @var LayoutRepository
	 */
	protected $layoutRepository;

	/**
	 * Inject the LayoutRepository
	 *
	 * @param LayoutRepository $layoutRepository
	 */
	public function injectLayoutRepository(LayoutRepository $layoutRepository) {
		$this->layoutRepository = $layoutRepository;
	}

	/**
	 * @var PhpSession
	 */
	protected $session;

	/**
	 * Command array on the form [tablename][uid][command] = value.
	 * This array may get additional data set internally based on clipboard commands send in clipboardCommandArray var!
	 *
	 * @var array
	 */
	protected $command;

	/**
	 * Clipboard command array. May trigger changes in "command"
	 *
	 * @var array
	 */
	protected $clipboardCommandArray;

	/**
	 * @var Clipboard
	 */
	protected $clipObj;

	/**
	 * Initializes the Action calls
	 *
	 * @return void
	 */
	public function initializeAction() {
		$this->command = GeneralUtility::_GP('cmd');
		$this->clipboardCommandArray = GeneralUtility::_GP('CB');
	}

	/**
	 * @param ViewInterface $view
	 */
	public function initializeView(ViewInterface $view) {
		parent::initializeView($view);
		$this->initClipboard();
		$view->assign('controller', 'Layout');
	}

	/**
	 * @throws NoSuchArgumentException
	 */
	public function indexAction() {
		$pageUid = (int) GeneralUtility::_GP('id');

		if (!($this->session instanceof PhpSession)) {
			if (version_compare(VersionNumberUtility::getCurrentTypo3Version(), '10.4.0', '<')) {
				/** @var PhpSession session */
				$this->session = $this->objectManager->get(PhpSession::class);
			} else {
				$this->session = GeneralUtility::makeInstance(PhpSession::class);
			}

			$this->session->setSessionKey('sg_mail_controller_session');
		} else {
			$this->session->setSessionKey('sg_mail_controller_session');
		}

		if ($this->request->hasArgument('controller')) {
			$this->session->setDataByKey('mode', $this->request->getArgument('controller'));
		}

		// create doc header component
		$pageInfo = BackendUtility::readPageAccess($pageUid, $GLOBALS['BE_USER']->getPagePermsClause(1));
		if ($pageInfo && (int) $pageInfo['is_siteroot'] === 1) {
			$layouts = $this->layoutRepository->findByPidForModule($pageUid);
			$this->view->assign('layouts', $layouts);
			$pasteData = $this->clipObj->elFromTable('tx_sgmail_domain_model_layout');
			if (\count($pasteData)) {
				$pasteButton = [
					'message' => $this->clipObj->confirmMsgText('pages', $pageInfo, 'into', $pasteData),
					'url' => $this->clipObj->pasteUrl('', $pageUid)
				];
				$this->view->assign('pasteButton', $pasteButton);
			}
		} else {
			$this->view->assign('pages', BackendService::getPages());
		}
		$this->docHeaderComponent = GeneralUtility::makeInstance(DocHeaderComponent::class);
		if ($pageInfo === FALSE) {
			$pageInfo = ['uid' => $pageUid];
		}
		$this->docHeaderComponent->setMetaInformation($pageInfo);
		BackendService::makeButtons($this->docHeaderComponent, $this->request);
		$this->view->assign('pageUid', $pageUid);
		$this->view->assign('docHeader', $this->docHeaderComponent->docHeaderContent());
	}

	/**
	 * Clipboard pasting and deleting.
	 *
	 * @return void
	 * @throws \InvalidArgumentException
	 */
	protected function initClipboard() {
		if (!$this->clipObj) {
			$this->clipObj = GeneralUtility::makeInstance(Clipboard::class);
		}
		$this->clipObj->initializeClipboard();
		if (\is_array($this->clipboardCommandArray)) {
			if ($this->clipboardCommandArray['paste']) {
				$this->clipObj->setCurrentPad($this->clipboardCommandArray['pad']);
				$this->command = $this->clipObj->makePasteCmdArray(
					$this->clipboardCommandArray['paste'],
					$this->command,
					$this->clipboardCommandArray['update'] ?? NULL
				);
			}
			if ($this->clipboardCommandArray['delete']) {
				$this->clipObj->setCurrentPad($this->clipboardCommandArray['pad']);
				$this->command = $this->clipObj->makeDeleteCmdArray($this->command);
			}
			if ($this->clipboardCommandArray['el']) {
				$this->clipboardCommandArray['setP'] = 'normal';
				$this->clipObj->setCmd($this->clipboardCommandArray);
				$this->clipObj->cleanCurrent();
				$this->clipObj->endClipboard();
			}
		}
	}
}
